#!/usr/bin/env python

import sys
import os
import logging
from shutil import move, rmtree

from fs_transformation import FSTransformation
import json

DATA_FOLDER = "data"

def get_datasource_id_to_nametype(datasource_dict):
  res = {}
  for next_ds in datasource_dict:
    res[next_ds["id"]] = [next_ds["connector"], next_ds["type"]]
  return res


def rename_connectors(fusion_home_old, fusion_home_new, datasources):
  old_data = os.path.join(fusion_home_old, DATA_FOLDER)
  if not os.path.exists(old_data):
    sys.exit("Old Fusion data folder doesn't exist at path '{0}'.".format(old_data))
  new_data = os.path.join(fusion_home_new, DATA_FOLDER)
  if not os.path.exists(fusion_home_new):
    sys.exit("New Fusion home folder doesn't exist at path '{0}'.".format(fusion_home_new))
  if not os.path.exists(new_data):
    os.makedirs(new_data)

  container_dir = os.path.join(new_data, 'connectors', 'container')
  crawldb_dir = os.path.join(new_data, 'connectors', 'crawldb')

  ds_affected = ['lucid.anda', 'lucid.fs']
  fs_old_crawldb_dir = os.path.join(old_data, 'connectors', 'crawldb', "lucid.fs")
  if not os.path.exists(fs_old_crawldb_dir):
    logging.info("Old Fusion crawlDb folder %s doesn't exist for lucid.fs", fs_old_crawldb_dir)
    ds_affected.remove("lucid.fs")
  anda_old_crawldb_dir = os.path.join(old_data, 'connectors', 'crawldb', "lucid.anda")
  if not os.path.exists(anda_old_crawldb_dir):
    logging.info("Old Fusion crawlDb folder %s doesn't exist for lucid.anda", anda_old_crawldb_dir)
    ds_affected.remove("lucid.anda")
  rename_plugins_in_dir(crawldb_dir, datasources, ds_affected)
  if 'lucid.anda' in ds_affected:
    rename_plugins_in_dir(container_dir, datasources, ['lucid.anda'])


def rename_plugins_in_dir(dir, datasources, ds_affected):
  '''
  Rename all lucid.anda or lucid.fs plugins to lucid.{type}
  which is part of the migration to connectors 3.1.0 which are
  split out now.

  :param dir: Is either the data/connectors/container folder or the data/connectors/crawldb folder.
  :param datasources: The datasources object from zookeeper.
  :param ds_affected: A list of names of the datasources we need to rename.
  '''

  # If the base directory doesn't exist, nothing to do.
  # This is possible only on multi-node Fusion.
  if not os.path.exists(dir):
    logging.info("Directory %s does not exist on this Fusion node. Nothing to rename.", dir)
    return

  # Go through each datasource and check if there is a directory
  # in this "dir" to rename.
  datasource_id_to_nametype = get_datasource_id_to_nametype(datasources)
  for next_ds_id in datasource_id_to_nametype.keys():
    next_nametype = datasource_id_to_nametype[next_ds_id]
    if next_nametype[0] in ds_affected:
      old_plugin_dir = os.path.join(dir, next_nametype[0])
      new_plugin_dir = os.path.join(dir, 'lucid.{}'.format(next_nametype[1]))
      if not os.path.isdir(new_plugin_dir):
        os.mkdir(new_plugin_dir)
      to_move = os.path.join(old_plugin_dir, next_ds_id)
      logging.info("Moving %s to %s", to_move, new_plugin_dir)
      move(to_move, new_plugin_dir)
  to_del = os.path.join(dir, "lucid.anda")
  if os.path.isdir(to_del):
    logging.info("Deleting old %s directory", to_del)
    rmtree(to_del)
  to_del = os.path.join(dir, "lucid.fs")
  if os.path.isdir(to_del):
    logging.info("Deleting old %s direcotry", to_del)
    rmtree(to_del)

if __name__ == "__main__":
  fsTran = FSTransformation("This transformation will rename the lucid.anda connectors to their new broken \
        out names such as lucid.anda/box will become lucid.box/box")
  fusion_home_old = fsTran.get_old_fusion()
  fusion_home_new = fsTran.get_new_fusion()
  datasources = fsTran.read_full_zk_dump("/core/connectors/datasources/")
  logging.debug("Datasources from zk_dump %s", json.dumps(datasources))
  rename_connectors(fusion_home_old, fusion_home_new, datasources)
