#!/usr/bin/env python

import logging
import json

DATASOURCE_TYPE="datasource"
PARSER_TYPE="parser"
INDEX_PIPELINE_TYPE="index-pipeline"
QUERY_PIPELINE_TYPE="query-pipeline"
COLLECTION_TYPE="collection"
INDEX_PROFILE_TYPE="index-profile"
QUERY_PROFILE_TYPE="query-profile"
UNSPECIFIED_BLOB_TYPE="other"
BLOB_TYPE="blob"
OPEN_NLP_TYPE="model:open-nlp"

DEPENDS_ON_LINK_TYPE="dependsOn"
RELATES_TO_LINK_TYPE="relatesTo"


def createLink(subject_frt, subject, object_frt, object, linkType):
    '''

    :param subject_frt: fusion resource type for the subject
    :param subject: fusion resource id for the subject
    :param object_frt: fusion resource type for the object
    :param object: fusion resource id for the object
    :param linkType: the type of link between the subject and the object
    :return:
    '''
    link = {
        "subject":"{}:{}".format(subject_frt, subject),
        "object":"{}:{}".format(object_frt, object),
        "linkType":linkType
    }

    logging.debug("defined link {}".format(json.dumps(link)))

    return link


def createPipelineLink(datasource):
    '''
    link a datasource to its index pipeline
    :param datasource:
    :return:
    '''
    datasource_id = datasource["id"]
    pipeline_id = datasource["pipeline"]
    link = createLink("datasource",datasource_id,INDEX_PIPELINE_TYPE,pipeline_id,DEPENDS_ON_LINK_TYPE)
    return link


def createParserLink(datasource):
    '''
    link a data source to its parser
    :param datasource:
    :return:
    '''
    datasource_id = datasource["id"]
    parser_id = datasource["parserId"]
    link = createLink(DATASOURCE_TYPE,datasource_id, PARSER_TYPE,parser_id,DEPENDS_ON_LINK_TYPE)
    return link

def createBlobLink(datasource):
    '''
    link a data source to the blob
    :param datasource:
    :return:
    '''
    datasource_id = datasource["id"]
    blob_id = datasource["properties"]["fileId"]
    link = createLink(DATASOURCE_TYPE,datasource_id, BLOB_TYPE,blob_id,DEPENDS_ON_LINK_TYPE)
    return link


def createCollectionLink(datasource):
    '''
    link data source to its collection
    :param datasource:
    :return:
    '''
    datasource_id = datasource["id"]
    collection_id = datasource["properties"]["collection"]
    link = createLink(DATASOURCE_TYPE,datasource_id, COLLECTION_TYPE,collection_id,RELATES_TO_LINK_TYPE)
    return link


def create_links_from_data_source(datasource):
    '''
    create a link from a data source
    :param datasource:
    :return: array of links from datasource
    '''
    links=[]
    indexPipelineLink = createPipelineLink(datasource)
    links.append(indexPipelineLink)
    if "parserId" in datasource:
        parserLink = createParserLink(datasource)
        links.append(parserLink)
        logging.debug("datasource looks like {}".format(json.dumps(datasource)))
    if "fileupload" == datasource["type"] and "fileId" in datasource["properties"]:
        logging.debug("creating a blob link from datasource {}".format(json.dumps(datasource)))
        blobLink = createBlobLink(datasource)
        links.append(blobLink)
    collectionLink=createCollectionLink(datasource)
    links.append(collectionLink)
    return links

def create_links_from_profile(profile,collection_id, profile_frt,pipeline_frt):
    '''
    create a link from a profile to the  pipeline and the associated collection
    :param profile:
    :param collection_id:
    :param profile_frt: profile fusion resource type
    :param pipeline_frt: pipeline fusion resource type
    :return:
    '''
    logging.debug("extracting links for profile in collection_id is {} from data {}".format(collection_id, profile))
    links=[]
    for profile_name in profile["pipelines"].keys():
        logging.debug("looking at profile {}".format(profile_name))
        pipeline_id = profile["pipelines"][profile_name]
        link = createLink(profile_frt,profile_name,pipeline_frt,pipeline_id,DEPENDS_ON_LINK_TYPE)
        links.append(link)
        link = createLink(profile_frt,profile_name,COLLECTION_TYPE,collection_id, RELATES_TO_LINK_TYPE)
        links.append(link)

    logging.debug("links generated from profiles are {}".format(json.dumps(links)))
    return links


def report_missing_field(pipeline_frt, pipeline_id, key, stage_type, data):
    '''
    report the missing field of the stage
    :param pipeline_frt:
    :param pipeline_id:
    :param key:
    :param stage_type
    :param data:
    :return:
    '''
    logging.info("WARNING: The {} with id {} is missing the key {} in the stage of type {} - no link generated -- data is {}".format(pipeline_frt, pipeline_id, key, stage_type, json.dumps(data)))


def create_links_from_pipeline(pipeline, pipeline_frt):
    links = []
    stages = pipeline["stages"]
    for stage in stages:
        stage_links = []

        if "type" not in stage:
            logging.info("WARNING: The {} with id {} is missing its type in the stage {} - no links generated for the stage".format(pipeline_frt, pipeline["id"], json.dumps(stage)))
            continue

        stage_type = stage["type"]
        target_resource01 = None
        target_resource02 = None

        if "exclusion-filter" == stage_type :
            if "location" in stage:
                target_resource01 = stage["location"]
                stage_links.append(createLink(pipeline_frt,pipeline["id"],UNSPECIFIED_BLOB_TYPE,target_resource01,DEPENDS_ON_LINK_TYPE))
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"location", stage_type, stage)

        elif "nlp-extractor" == stage_type :
            if "rules" in stage:
                rules = stage["rules"]
                for rule in rules:
                    if "sentenceModelLocation" in rule:
                        target_resource01 = rule["sentenceModelLocation"]
                        stage_links.append(createLink(pipeline_frt,pipeline["id"],OPEN_NLP_TYPE,target_resource01,DEPENDS_ON_LINK_TYPE))
                    else:
                        report_missing_field(pipeline_frt,pipeline["id"],"sentenceModelLocation", stage_type, rule)
                    if "tokenizerModelLocation" in rule :
                        target_resource02 = rule["tokenizerModelLocation"]
                        stage_links.append(createLink(pipeline_frt,pipeline["id"],OPEN_NLP_TYPE,target_resource02,DEPENDS_ON_LINK_TYPE))
                    else:
                        report_missing_field(pipeline_frt,pipeline["id"],"tokenizerModelLocation", stage_type, rule)
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"rules", stage_type, stage)

        elif "field-parser" == stage_type :
            if "parserId" in stage:
                target_resource01 = stage["parserId"]
                stage_links.append(createLink(pipeline_frt,pipeline["id"],PARSER_TYPE,target_resource01,DEPENDS_ON_LINK_TYPE))
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"parserId", stage_type, stage)

        elif "part-of-speech" == stage_type :
            if "posModel" in stage:
                target_resource01 = stage["posModel"]
                stage_links.append(createLink(pipeline_frt,pipeline["id"],OPEN_NLP_TYPE,target_resource01,DEPENDS_ON_LINK_TYPE))
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"posModel", stage_type, stage)

            if "tokenizerModel" in stage:
                target_resource02 = stage["tokenizerModel"]
                stage_links.append(createLink(pipeline_frt,pipeline["id"],OPEN_NLP_TYPE,target_resource02,DEPENDS_ON_LINK_TYPE))
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"tokenizerModel", stage_type, stage)

        elif "sentence-detection" == stage_type :
            if "sentenceModel" in stage:
                target_resource01 = stage["sentenceModel"]
                stage_links.append(createLink(pipeline_frt,pipeline["id"],OPEN_NLP_TYPE,target_resource01,DEPENDS_ON_LINK_TYPE))
            else:
                report_missing_field(pipeline_frt,pipeline["id"],"sentenceModel", stage_type, stage)

        else:
            logging.debug("No links created for stage {} inside {} {}".format(stage_type, pipeline_frt, pipeline["id"]))

        if 0 < len(stage_links) :
            logging.debug("for the part of {} {} from stage {} - created the links {}".format(pipeline_frt, pipeline["id"],stage["type"],json.dumps(stage_links)))
        else:
            logging.debug("no links created from {} {} for stage{}".format(pipeline_frt, pipeline["id"],stage["type"]))

        links.extend(stage_links)

    if 0 < len(links) :
        logging.debug("for {} {} created the links {}".format(pipeline_frt, pipeline["id"], json.dumps(links)))
    else:
        logging.debug("no links created for {} {}".format(pipeline_frt, pipeline["id"]))

    return links