#!/usr/bin/env python

import logging
import re
import uuid
 
from oie_transformation import OIETransformation
from schedule_helper import convertScheduleIntoJob
 
def map_agg_defs(aggs):
    agg_defs = {}
    for a in aggs:
        agg_defs[a["id"]] = a
    return agg_defs

def extract_main_collection_name(coll):
    match = re.match(r'(.*)_signals',coll)
    return coll if match is None else match.group(1)

def remove_agg_schedules(schedules):
    updated = []
    agg_uri_regex = re.compile(r'service://aggregator/jobs/(.*)/(.*)')
    for schedule in schedules:
        match = agg_uri_regex.match(schedule["callParams"]["uri"])
        if match is None:
            updated.append(schedule)
    return updated

def get_agg_schedules(schedules):
    agg_uri_regex = re.compile(r'service://aggregator/jobs/(.*)/(.*)')
    agg_schedules = {}
    for schedule in schedules:
        match = agg_uri_regex.match(schedule["callParams"]["uri"])
        if match is not None:
            agg_def = match.group(2)
            if agg_def not in agg_schedules:
                agg_schedules[agg_def] = []
            agg_schedules[agg_def].append(schedule)
    return agg_schedules

def get_collection_from_schedule(sch):
    match = re.match(r'service://aggregator/jobs/(.*)/(.*)', sch["callParams"]["uri"])
    return match.group(1)

def old_create_job_from_schedule(spark_job_id, sch):
    '''
    this function will be removed once PR is good
    :param spark_job_id:
    :param sch:
    :return:
    '''
    triggers = []
    enabled = sch["active"]
    trigger = {"enabled":enabled, "interval":sch["interval"], "timeUnit":sch["repeatUnit"].lower(), "type":"interval"}
    if "startTime" in sch:
        trigger["startTime"] = sch["startTime"]
    triggers.append(trigger)
    return {"resource":"spark:{}".format(spark_job_id), "enabled":True, "triggers":triggers, "default":False}


def convertSparkJobIdIntoSparkJobResource(spark_job_id):
    '''
    Convert the spark job id into a fusion resource style uri for a spark job
    :param spark_job_id:
    :return:  a resource string based on the id provided in the schedule uri, or None if no id is present
    '''


    if spark_job_id is None or 0 == len(spark_job_id):
        logging.error("The  spark_job_id may not be null or empty")
        return None

    logging.debug("The spark_job_id was {}".format(spark_job_id))
    resource="spark:{}".format(spark_job_id)
    logging.debug("The resource is {}".format(resource))
    return resource


def create_job_from_schedule(spark_job_id, sch):
    return convertScheduleIntoJob(sch,convertSparkJobIdIntoSparkJobResource,spark_job_id)


def create_link(subject, prefix, object, linkType):
    return {"subject":"group:{}".format(subject), "object":"{}:{}".format(prefix, object), "linkType":linkType}

def create_group(collection_id):
    group_id = str(uuid.uuid4())
    return { "id":group_id, "name":"signals-{}".format(collection_id), "isSystem":True, "description":"Collection '{}' objects for signals support".format(collection_id) }

def get_signals_jobs(collection_id, spark_jobs):
    jobs = []
    for j in spark_jobs:
        if j.get("type","") == "aggregation":
            if j.get("inputCollection", "") in (collection_id, "{}_signals".format(collection_id)):
                jobs.append(j["id"])
    return jobs

if __name__ == "__main__":
    oieTran = OIETransformation("Migrate aggregation schedules to Spark jobs")

    schedules = oieTran.read_full_zk_dump("/core/schedules/")
    agg_schedules = get_agg_schedules(schedules)
    objects = oieTran.get_fusion_objects()
    agg_defs = map_agg_defs(objects.get("aggregations",[]))

    # convert aggregation definitions to spark jobs, which need an input collection (from schedule) and the definition
    jobs = objects.get("jobs", [])
    spark_jobs = []
    counter = 0
    signals_job_names = {}
    for d in agg_schedules:
        if d not in agg_defs:
            logging.info("Aggregation definition {} not found! Can't migrate schedules: {}".format(d, agg_schedules[d]))
            continue

        agg_def = agg_defs[d]
        for sch in agg_schedules[d]:
            c = get_collection_from_schedule(sch)
            main_coll = extract_main_collection_name(c)
            spark_job_id = "{}_{}".format(main_coll,d)
            if d == "click-signals-default":
                spark_job_id = "{}_click_signals_aggregation".format(main_coll)
            elif sch["creatorType"] == "system" and (d.startswith("dailyMetrics") or d.startswith("hourlyMetrics")):
                spark_job_id = d # just use the aggregation job name for this one

            spark_jobs.append({"id":spark_job_id, "type":"aggregation", "definition":agg_def, "inputCollection":c })
            jobs.append(create_job_from_schedule(spark_job_id, sch))
            counter = counter + 1
            signals_job_names[main_coll] = spark_job_id

    objects["sparkJobs"] = spark_jobs
    objects["jobs"] = jobs

    if "schedules" in objects:
        objects["schedules"] = remove_agg_schedules(objects["schedules"])

    if "aggregations" in objects:
        del objects["aggregations"]

    # build signals group/links for collections with signals
    object_groups = objects.get("objectGroups", [])
    links = objects.get("links", [])

    collections = oieTran.read_full_zk_dump("/core/collections/")
    for c in collections:
        if "features" not in c or "signals" not in c["features"]:
            continue

        collection_id = c["id"]
        group = create_group(collection_id)
        object_groups.append(group)
        group_id = group["id"]
        links.append(create_link(group_id, "collection", collection_id, "relatesTo"))
        links.append(create_link(group_id, "collection", "{}_signals".format(collection_id), "hasPart"))
        links.append(create_link(group_id, "collection", "{}_signals_aggr".format(collection_id), "hasPart"))
        for j in get_signals_jobs(collection_id, spark_jobs):
            links.append(create_link(group_id, "spark", j, "hasPart"))

    objects["objectGroups"] = object_groups
    objects["links"] = links

    oieTran.save_output()
    logging.info("Migrated {} aggregation job schedules to Spark job schedules.".format(counter))
