#!/usr/bin/env python

import logging
import json

from oie_transformation import OIETransformation
from link_helper import create_links_from_data_source
from link_helper import create_links_from_profile
from link_helper import create_links_from_pipeline

from link_helper import INDEX_PIPELINE_TYPE
from link_helper import QUERY_PIPELINE_TYPE
from link_helper import INDEX_PROFILE_TYPE
from link_helper import QUERY_PROFILE_TYPE




def create_links_from_datasources(data_sources):
    '''
    create links from a set of datasources
    :param data_sources:
    :return: array of links for the all of the data sources
    '''
    links=[]
    for data_source in data_sources:
        dataSourceId = data_source["id"]
        logging.debug("Examining the datasource with id {} for links to its parser and index pipeline".format(dataSourceId))
        logging.debug("The datasource looks like {}".format(json.dumps(data_source)))
        data_source_links = create_links_from_data_source(data_source)
        logging.debug("Created links from the datasource with id {} into the links {}".format(dataSourceId,json.dumps(data_source_links)))
        links.extend(data_source_links)

    return links



def create_links_from_collections(oieTran, collections):
    '''
    create links from collections
    :param oieTran:
    :param collections:
    :return:
    '''
    profile_oriented_links=[]
    for collection in collections:
        collection_id = collection["id"]
        logging.debug("collection_id is {}".format(collection_id))
        index_profiles = oieTran.read_full_zk_dump("/core/index-profiles/" + collection_id)

        if 0 < len(index_profiles):
            logging.debug("extracting links for index profile in collection_id is {}".format(collection_id))
            links=create_links_from_profile(index_profiles[0],collection_id, INDEX_PROFILE_TYPE, INDEX_PIPELINE_TYPE)
            profile_oriented_links.extend(links)
        query_profiles = oieTran.read_full_zk_dump("/core/query-profiles/" + collection_id )
        if 0 < len(query_profiles):
            logging.debug("extracting links for query profile in collection_id is {}".format(collection_id))
            links=create_links_from_profile(query_profiles[0],collection_id, QUERY_PROFILE_TYPE, QUERY_PIPELINE_TYPE)
            profile_oriented_links.extend(links)

    return profile_oriented_links


def do_create_links_from_pipelines(pipelines, pipeline_frt):
    '''
    create links between pipelines and their objects such as stages
    :param pipelines:
    :param pipeline_frt:
    :return:
    '''
    links = []

    for pipeline in pipelines:
        pipeline_links = create_links_from_pipeline(pipeline,pipeline_frt)
        links.extend(pipeline_links)
    return links


def create_links_from_pipelines(oieTran):
    '''
    create links for pipelines
    :param pipelines:
    :return:
    '''
    links = []
    index_pipelines = oieTran.read_full_zk_dump("/core/index-pipelines/")
    links.extend(do_create_links_from_pipelines(index_pipelines, INDEX_PIPELINE_TYPE))
    query_pipelines = oieTran.read_full_zk_dump("/core/query-pipelines/")
    links.extend(do_create_links_from_pipelines(query_pipelines, QUERY_PIPELINE_TYPE))
    return links


if __name__=="__main__":
    # we are testing if jobs are already in the import/export file
    oieTran = OIETransformation("Link Transformation")
    objects = oieTran.get_fusion_objects()
    if "links" in objects:
        links = objects["links"]
        logging.debug("Found existing {} links".format(len(links)))
    else:
        links = []
        logging.info("Found no existing links")
    generated_links = []
    # Now retrieve from zookeeper all of the data source  and convert them into links
    data_sources = oieTran.read_full_zk_dump("/core/connectors/datasources/")
    generated_links.extend(create_links_from_datasources(data_sources))

    # get the collections so that we can derive the links of profiles
    collections = oieTran.read_full_zk_dump("/core/collections/")
    generated_links.extend(create_links_from_collections(oieTran, collections))

    # get the pipelines for link derivation

    generated_links.extend(create_links_from_pipelines(oieTran))

    for generated_link in generated_links:
        logging.info("generated link {}".format(json.dumps(generated_link)))

    links.extend(generated_links)
    objects["links"] = links
    oieTran.save_output()