#!/usr/bin/env python

import os
import logging
from fs_transformation import FSTransformation
import json

from java.net import URL
from java.io import File
from java.io import IOException
from org.apache.commons.io import FileUtils

from org.apache.http.client.methods import HttpPut
from org.apache.http.entity import ContentType,FileEntity
from org.apache.http.impl.client import HttpClientBuilder
from org.apache.http.client.config import RequestConfig

# The following are connector plugin zips that were available on 3.0.x but
# are not on the default 3.1.x distribution.
BASE_DOWNLOAD_URL = "https://download.lucidworks.com/fusion-3.1.0"
NON_DEFAULT_URLS = {
  "azure": "{}/lucid.azure-3.1.0.zip",
  "box": "{}/lucid.box-3.1.0.zip",
  "cmis": "{}/lucid.cmis-3.1.0.zip",
  "couchbase": "{}/lucid.couchbase-3.1.0.zip",
  "dropbox": "{}/lucid.dropbox-3.1.0.zip",
  "drupal": "{}/lucid.drupal-3.1.0.zip",
  "ftp": "{}/lucid.ftp-3.1.0.zip",
  "github": "{}/lucid.github-3.1.0.zip",
  "googledrive": "{}/lucid.googledrive-3.1.0.zip",
  "hadoop-apache2": "{}/lucid.hadoop-apache2-3.1.0.zip",
  "hadoop-cloudera": "{}/lucid.hadoop-cloudera-3.1.0.zip",
  "hadoop-hortonworks": "{}/lucid.hadoop-hortonworks-3.1.0.zip",
  "hadoop-mapr": "{}/lucid.hadoop-mapr-3.1.0.zip",
  "hdfs": "{}/lucid.hdfs-3.1.0.zip",
  "javascript": "{}/lucid.javascript-3.1.0.zip",
  "jira": "{}/lucid.jira-3.1.0.zip",
  "jive": "{}/lucid.jive-3.1.0.zip",
  "mongodb": "{}/lucid.mongodb-3.1.0.zip",
  "push": "{}/lucid.push-3.1.0.zip",
  "s3": "{}/lucid.s3-3.1.0.zip",
  "salesforce": "{}/lucid.salesforce-3.1.0.zip",
  "servicenow": "{}/lucid.servicenow-3.1.0.zip",
  "sharepoint": "{}/lucid.sharepoint-3.1.0.zip",
  "slack": "{}/lucid.slack-3.1.0.zip",
  "smb": "{}/lucid.smb-3.1.0.zip",
  "solr": "{}/lucid.solr-3.1.0.zip",
  "solrxml": "{}/lucid.solrxml-3.1.0.zip",
  "subversion": "{}/lucid.subversion-3.1.0.zip",
  "twitter-search": "{}/lucid.twitter-search-3.1.0.zip",
  "twitter-stream": "{}/lucid.twitter-stream-3.1.0.zip",
  "zendesk": "{}/lucid.zendesk-3.1.0.zip"
}

def get_datasource_id_to_nametype(datasource_dict):
  res = {}
  for next_ds in datasource_dict:
    res[next_ds["id"]] = [next_ds["connector"], next_ds["type"]]
  return res

def download_missing_connectors(datasources, tmp_folder, api_connect):
  '''
  Previously all connectors were installed by default.
  Now they are not. So we need to download them when needed.
  '''
  failed_to_download = []
  uploaded = set()
  datasource_id_to_nametype = get_datasource_id_to_nametype(datasources)
  for next_ds_id in datasource_id_to_nametype.keys():
    next_nametype = datasource_id_to_nametype[next_ds_id]
    logging.info("Checking to see if we need to install datasource %s's connector %s of type %s",
                 next_ds_id, next_nametype[0], next_nametype[1])
    if NON_DEFAULT_URLS.has_key(next_nametype[1]):
      next_url = NON_DEFAULT_URLS[next_nametype[1]].format(BASE_DOWNLOAD_URL)
      if next_url in uploaded:
        continue
      next_file_name = os.path.basename(next_url)
      next_file_path = os.path.join(tmp_folder, next_file_name)
      logging.info("Downloading %s to %s...", next_url, next_file_path)
      try:
        uploaded.add(next_url)
        FileUtils.copyURLToFile(URL(next_url), File(next_file_path), 100000, 100000)
        logging.info("Done downloading connector plugin %s. Uploading to Fusion...", next_file_name)
        entity = FileEntity(File(next_file_path),
                          ContentType.create("application/zip"));
        request = HttpPut("http://{0}/api/v1/blobs/lucid.{1}?resourceType=plugin:connector".format(api_connect, next_nametype[1]));
        request.setEntity(entity);
        params = RequestConfig.custom().setConnectTimeout(100000).setSocketTimeout(100000).setConnectionRequestTimeout(100000).build()
        client = HttpClientBuilder.create().setDefaultRequestConfig(params).build()
        response = client.execute(request);
        logging.info("Connector plugin upload response: %s", response)
      except IOException, e:
        logging.error("Failed to download connector plugin %s due to error %s.", next_file_name, e)
        failed_to_download.append(next_url)
  if len(failed_to_download) > 0:
    msg = "\n"
    for next_failed_plugin_zip in failed_to_download:
      msg += "\n{}".format(next_failed_plugin_zip)
    logging.info("\n\n\n\n NOTICE! \n\n\nThis version of Fusion doesn't come out of box with several connectors that " \
        "you had in the previous version. We tried to download them automatically, but couldn't. You will need to " \
        "download the following zip files manually and upload them to Fusion. \n(See " \
        "https://doc.lucidworks.com/fusion/3.1/Connectors-and-Datasources-Reference.html#installing-a-connector for " \
        "instructions on how to accomplish this): %s\n", msg)

if __name__ == "__main__":
  fs_tran = FSTransformation("Download the connectors that were on the system previously \
          but are not part of Fusion's default connectors.")
  fusion_home_new = fs_tran.get_new_fusion()
  fusion_home_old = fs_tran.get_old_fusion()
  api_connect = fs_tran.get_api_connect()
  tmp_folder = os.path.join(fusion_home_old, "var", "upgrade", "tmp")
  datasources = fs_tran.read_full_zk_dump("/core/connectors/datasources/")
  logging.debug("Datasources from zk_dump %s", json.dumps(datasources))
  download_missing_connectors(datasources, tmp_folder, api_connect)
