 #!/usr/bin/env python

import json
import sys
import os
import argparse
import logging
import re

from base_transformation import BaseTransformation

class ZKTransformation(BaseTransformation):
    def __init__(self, description):
        logging.basicConfig(level=logging.INFO,
                     format="%(asctime)s - %(name)s - %(filename)s:%(lineno)d - %(levelname)s - %(message)s")
        parser = argparse.ArgumentParser(description=description)
        parser.add_argument("--input-file", required=True, help="File to modify")
        parser.add_argument("--output-file", required=True, help="File to output")
        parser.add_argument("--props-file", required=True, help="Properties file")
        parser.add_argument("--full-zk-dump", required=True, help="Full zk dump file")
        args = parser.parse_args()
        self.input_file = args.input_file
        self.output_file = args.output_file
        full_zk_dump = args.full_zk_dump
        props_file = args.props_file
        BaseTransformation.__init__(self, props_file, full_zk_dump)
        if not os.path.exists(self.input_file):
            sys.exit("Input file does not exist at path '{0}'.".format(self.input_file))
        if os.path.exists(self.output_file):
            sys.exit("Output file already exists at path '{0}'.".format(self.output_file))
        self.zk_properties = None

    def update_json(self, pattern, line_processor, error_if_not_found=True):
        found = False
        with open(self.input_file) as f:
            with open(self.output_file, 'w') as outfile:
                for line in f:
                    znode = json.loads(line)
                    if pattern.match(znode["path"]) is not None:
                        if not znode["isUTF8"]:
                            sys.exit("Trying to tranform base64 data: {0}".format(line))
                        deser_payload = json.loads(znode["data"])
                        znode["data"] = json.dumps(line_processor(self, deser_payload))
                        found = True
                    outfile.write(json.dumps(znode) + '\n')
        if not found and error_if_not_found:
            sys.exit("Can't find path '{0}'.".format(pattern.pattern))

    def update_exact_path_json(self, path, json_processor, error_if_not_found=True):
        new_chroot = self.get_new_chroot()
        p = re.compile(re.escape(new_chroot + path) + "$")
        self.update_json(p, json_processor, error_if_not_found)

    def update_all_nodes(self, node_processor, error_if_not_found=True):
        found = False
        with open(self.input_file) as f:
            with open(self.output_file, 'w') as outfile:
                for line in f:
                    znode = json.loads(line)
                    result = node_processor(self, znode)
                    found = found or result
                    outfile.write(json.dumps(znode) + '\n')
        if (not found and error_if_not_found):
            sys.exit("No lines were changed")

    def update_collections(self, json_processor, error_if_not_found=True):
        new_chroot = self.get_new_chroot()
        p = re.compile(re.escape(new_chroot + "/core/collections/"))
        self.update_json(p, json_processor, error_if_not_found)

    def update_datasources(self, json_processor, error_if_not_found=True):
        new_chroot = self.get_new_chroot()
        p = re.compile(re.escape(new_chroot + "/core/connectors/datasources/"))
        self.update_json(p, json_processor, error_if_not_found)

    def update_roles(self, json_processor, error_if_not_found=True):
        new_chroot = self.get_new_chroot()
        p = re.compile(re.escape(new_chroot + "/proxy/role/"))
        self.update_json(p, json_processor, error_if_not_found)

    def update_users(self, json_processor, error_if_not_found=True):
        new_chroot = self.get_new_chroot()
        p = re.compile(re.escape(new_chroot + "/proxy/user/"))
        self.update_json(p, json_processor, error_if_not_found)
